<?php

declare(strict_types=1);

namespace Drupal\node_swapper\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\node\Entity\Node;
use Drupal\path_alias\Entity\PathAlias;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Database\Connection;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * The Node Swapper confirm form.
 */
final class NodeSwapperConfirmForm extends ConfirmFormBase {

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * Time interface.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new NodeSwapperConfirmForm.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection service.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   The alias manager service.
   * @param \Drupal\Component\DateTime\TimeInterface $time
   *   The time interface.
   * @param \Drupal\Core\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager interface.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    RequestStack $request_stack,
    Connection $connection,
    AliasManagerInterface $alias_manager,
    TimeInterface $time,
    EntityTypeManagerInterface $entity_type_manager,
  ) {
    $this->configFactory = $config_factory;
    $this->requestStack = $request_stack;
    $this->database = $connection;
    $this->aliasManager = $alias_manager;
    $this->time = $time;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('request_stack'),
      $container->get('database'),
      $container->get('path_alias.manager'),
      $container->get('datetime.time'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * ID of the item to swap from.
   *
   * @var int
   */
  protected $src;

  /**
   * ID of the item to swap to.
   *
   * @var int
   */
  protected $dest;

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'node_swapper_node_swapper_confirm';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion(): TranslatableMarkup {
    return $this->t('Are you sure you want to do this?');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl(): Url {
    return new Url('node_swapper.node_swapper');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $src = '', string $dest = '') {
    $src_node = Node::load($src);
    $dest_node = Node::load($dest);
    $config = $this->configFactory->get('node_swapper.settings');
    $request = $this->requestStack->getCurrentRequest();
    $unpublish = $request->query->get('unpublish', $config->get('old_node_unpublish'));
    $suffix = $request->query->get('suffix', $config->get('old_node_suffix'));

    $message = '';
    $message .= "<p>This will point all alises and redirects of \"{$src_node->title->value}\" to \"{$dest_node->title->value}\".</p>";
    $message .= "<p>In addition, <b>ALL</b> aliases of \"New Node\" will be deleted. Redirects will not be deleted.</p>";

    $form['notice'] = [
      '#type' => 'markup',
      '#markup' => $message,
    ];

    $form['transaction_summary'] = [
      '#prefix' => '<strong>Transaction Summary</strong>',
      '#type' => 'table',
      '#header' => [
        ['data' => ''],
        ['data' => $this->t('Old Node')],
        ['data' => $this->t('New Node')],
      ],
      '#rows' => [
        [
          'data' => [
            ['data' => $this->t('Name')],
            ['data' => $src_node->title->value],
            ['data' => $dest_node->title->value],
          ],
        ],
        [
          'data' => [
            ['data' => $this->t('Current URL')],
            ['data' => $this->aliasManager->getAliasByPath('/node/' . $src)],
            ['data' => $this->aliasManager->getAliasByPath('/node/' . $dest)],
          ],
        ],
        [
          'data' => [
            ['data' => $this->t('New URL')],
            ['data' => $this->aliasManager->getAliasByPath('/node/' . $src) . '-' . $suffix],
            ['data' => $this->aliasManager->getAliasByPath('/node/' . $src)],
          ],
        ],
        [
          'data' => [
            ['data' => $this->t('Aliases')],
            ['data' => $this->t('Will be assigned to the new node.')],
            ['data' => $this->t('All existing aliases will be replaced with those from the old node.')],
          ],
        ],
        [
          'data' => [
            ['data' => $this->t('Redirects')],
            ['data' => $this->t('Will be assigned to the new node.')],
            ['data' => $this->t('Existing redirects will be unchanged. Will inherit all existing aliases from the old node.')],
          ],
        ],
        [
          'data' => [
            ['data' => $this->t('Moderation State')],
            ['data' => $unpublish == '1' ? $this->t('Unpublished') : $this->t('Unchanged')],
            ['data' => $this->t('Unchanged')],
          ],
        ],
      ],
    ];

    $form['src'] = [
      '#type' => 'hidden',
      '#value' => $src_node->id(),
    ];

    $form['dest'] = [
      '#type' => 'hidden',
      '#value' => $dest_node->id(),
    ];

    $form['unpublish'] = [
      '#type' => 'hidden',
      '#value' => $unpublish,
    ];

    $form['suffix'] = [
      '#type' => 'hidden',
      '#value' => $suffix,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $src = $form_state->cleanValues()->getValue('src');
    $dest = $form_state->cleanValues()->getValue('dest');
    $suffix = $form_state->cleanValues()->getValue('suffix');
    $unpublish = $form_state->cleanValues()->getValue('unpublish');

    $src_aliases = $this->database->query('
      select id
      from {path_alias}
      where path = :source
    ', [':source' => '/node/' . $src])->fetchall();

    $src_redirects = $this->database->query('
      select rid
      from {redirect}
      where redirect_redirect__uri LIKE :source
    ', [':source' => '%node/' . $src . '%'])->fetchAll();

    $dest_aliases = $this->database->query('
      SELECT id
      FROM {path_alias}
      WHERE path = :source
    ', [':source' => '/node/' . $dest])->fetchAll();

    // Grab the latest source alias before we change anything.
    $latest_src_alias = $this->aliasManager->getAliasByPath("/node/$src");

    // Delete destination aliases.
    foreach ($dest_aliases as $da) {
      $alias = PathAlias::load($da->id);
      if (empty($alias)) {
        continue;
      }
      $alias->delete();
    }

    // Point source aliases to destination.
    foreach ($src_aliases as $sa) {
      $alias = PathAlias::load($sa->id);
      if (empty($alias)) {
        continue;
      }
      $alias->setPath("/node/$dest");
      $alias->save();
    }

    // Create a new alias for the old node so it includes the old node suffix.
    if (!empty($latest_src_alias)) {
      $alias_for_old_page = PathAlias::create();
      $alias_for_old_page->setPath("/node/$src");
      $alias_for_old_page->setAlias($latest_src_alias . '-' . $suffix);
      $alias_for_old_page->save();
    }

    // Point all source redirects to destination node.
    $redirect_storage = $this->entityTypeManager->getStorage('redirect');
    foreach ($src_redirects as $sr) {
      /**
       * @var \Drupal\redirect\Entity\Redirect $redirect
       */
      $redirect = $redirect_storage->load($sr->rid);
      if (empty($redirect)) {
        continue;
      }
      $redirect->setRedirect("/node/$dest");
      $redirect->save();
    }

    // Create new revisions for each node.
    $src_node = Node::load($src);
    $dest_node = Node::load($dest);

    $src_node->setNewRevision(TRUE);
    $src_node->setRevisionLogMessage("Node Swapper: Gave URL(s) to node #$dest");
    $src_node->setRevisionCreationTime($this->time->getCurrentTime());

    // Unpublish the old node if requested.
    if ($unpublish == '1') {
      $src_node->status = FALSE;
    }

    $src_node->save();

    $dest_node->setNewRevision(TRUE);
    $dest_node->setRevisionLogMessage("Node Swapper: Received URL(s) from node #$src");
    $dest_node->setRevisionCreationTime($this->time->getCurrentTime());
    $dest_node->save();

    $this->messenger()->addStatus($this->t('Node URLs successfully swapped'));
    $form_state->setRedirectUrl(new Url('node_swapper.node_swapper'));
  }

}
